@@ -22,33 +22,36 @@
 # SOFTWARE.
 
 import asyncio
-import websockets
-from time import time
-from uuid import uuid4
+from inspect import signature, Parameter
 from json import loads, dumps
 from logging import info, debug
+from time import time
 from traceback import print_exc
-from inspect import signature, Parameter
-from concurrent.futures import ThreadPoolExecutor
-from typing import Awaitable, List, Union, Tuple, Any
+from typing import Awaitable, List, Union, Tuple, Any, Dict
+from uuid import uuid4
+
+import websockets
 from websockets.exceptions import ConnectionClosedOK, ConnectionClosedError
 
-from .utils import Repr
+from .exceptions import *
 from .utils.convertors import Convertor
-from .entities import Client, User, Room, UserPreview, Message, BaseUser, Context
 from .config import apiUrl, heartbeatInterval, topPublicRoomsInterval, telemetryInterval
+from .entities import Client, User, Room, UserPreview, Message, BaseUser, Context
 from .utils.parsers import parse_sentence_to_tokens as parse_message, parse_word_to_token as parse_word
-from .exceptions import NoConnectionException, InvalidAccessToken, InvalidSize, NotEnoughArguments, CommandNotFound, MemberNotFound, CommandAlreadyDefined
-
 
 listeners = {}
 commands = {}
 
 
-def event(func: Awaitable):
+def event(func: Awaitable, *, name: str = None):
     """
     Create an event listener for dogehouse.
 
+    Args:
+        func (Awaitable, optional): Your callback (gets supplied automatically when you use the decorator).
+                                    Defaults to None.
+        name (str, optional): The event name. Defaults to the function name.
+
     Example:
         class Client(dogehouse.DogeClient):
             @dogehouse.event
@@ -58,16 +61,20 @@ def event(func: Awaitable):
         if __name__ == "__main__":
             Client("token", "refresh_token").run()
     """
-    listeners[func.__name__.lower()] = [func, False]
-    return func
+
+    def wrapper(_func: Awaitable):
+        listeners[(Convertor.convert_basic_types(name, str) if name else func.__name__).lower()] = [func, False]
+
+    return wrapper(func) if func else wrapper
 
 
-def command(func: Awaitable = None, *, name: str = None, cooldown: int = 0, aliases: List[str] = None):
+def command(func: Awaitable = None, *, name: str = None, cooldown: int = 0, aliases: List[str] = []):
     """
     Create a new DogeClient command, which will be handled by the dogehouse python library.
 
     Args:
-        func (Awaitable, optional): Your function (gets supplied automatically when you use the decorator). Defaults to None.
+        func (Awaitable, optional): Your callback (gets supplied automatically when you use the decorator).
+                                    Defaults to None.
         name (str, optional): The call name for your command. Defaults to the function name.
         cooldown (int, optional): The cooldown for the function usage per client. Defaults to 0.
         aliases (List[str], optional): A list of aliases which should trigger your command. Defaults to None.
@@ -81,24 +88,26 @@ def command(func: Awaitable = None, *, name: str = None, cooldown: int = 0, alia
         if __name__ == "__main__":
             Client("token", "refresh_token").run()
     """
-    def wrapper(func: Awaitable):
-        def save(name: str):
-            if name in commands:
-                raise CommandAlreadyDefined(f"Command `{name}` has already been defined by a name or alias!")
-            commands[name] = [func, False, int(cooldown)]
-        
-        save(str(name if name else func.__name__).lower())
-        if aliases:
-            any(save(alias) for alias in aliases)
-        
-        return func
+
+    def wrapper(_func: Awaitable):
+        def save(_name: str):
+            if _name in commands:
+                raise CommandAlreadyDefined(f"Command `{_name}` has already been defined by a name or alias!")
+            commands[_name] = [_func, False, int(cooldown)]
+
+        for cmd_name in (name if name else _func.__name__, *aliases):
+            save(Convertor.convert_basic_types(cmd_name, str).lower())
+
+        return _func
+
     return wrapper(func) if func else wrapper
 
 
 class DogeClient(Client):
     """Represents your Dogehouse client."""
 
-    def __init__(self, token: str, refresh_token: str, *, room: str = None, muted: bool = False, reconnect_voice: bool = False, prefix: Union[str, List[str]] = "!", telemetry = None):
+    def __init__(self, token: str, refresh_token: str, *, room: str = None, muted: bool = False,
+                 reconnect_voice: bool = False, prefix: Union[str, List[str]] = "!", telemetry=None):
         """
         Initialize your Dogehouse client
 
@@ -106,8 +115,9 @@ class DogeClient(Client):
             token (str): Your super secret client token.
             refresh_token (str): Your super secret client refresh token.
             room (int, optional): The room your client should join. Defaults to None.
-            muted (bool, optional): Wether or not the client should be muted. Defaults to False.
-            reconnect_voice (bool, optional): When the client disconnects from the voice server, should it try to reconnect. Defaults to False.
+            muted (bool, optional): Whether or not the client should be muted. Defaults to False.
+            reconnect_voice (bool, optional): When the client disconnects from the voice server, should it try to
+                                              reconnect. Defaults to False.
             prefix (List of strings or a string, optional): The bot prefix.
             telemetry (telemetry class, optional): The telemetry class that will be used. Defaults to None.
         """
@@ -126,10 +136,14 @@ class DogeClient(Client):
         self.__waiting_for_fetches = {}
         self.__command_cooldown_cache = {}
         self.telemetry = telemetry
-        
+
         if telemetry:
             asyncio.ensure_future(telemetry.start())
 
+    @property
+    def commands(self):
+        return self.__commands
+
     async def __fetch(self, op: str, data: dict):
         fetch = str(uuid4())
 
@@ -145,6 +159,7 @@ class DogeClient(Client):
 
     async def __main(self, loop):
         """This instance handles the websocket connections."""
+
         async def event_loop():
             async def execute_listener(listener: str, *args):
                 listener_name = listener.lower()
@@ -155,31 +170,32 @@ class DogeClient(Client):
 
                     if listener_name[3::] in self.__waiting_for:
                         for fetch_id in self.__waiting_for[listener_name[3::]]:
-                            self.__waiting_for_fetches[fetch_id] = [*self.__waiting_for_fetches[fetch_id],
-                                                                    list(args)] if fetch_id in self.__waiting_for_fetches else [list(args)]
+                            self.__waiting_for_fetches[fetch_id] = [*self.__waiting_for_fetches[fetch_id], list(args)] \
+                                if fetch_id in self.__waiting_for_fetches else [list(args)]
 
             async def execute_command(command_name: str, ctx: Context, *args):
-                command = self.__commands.get(command_name.lower())
-            
-                if command:
+                _command = self.__commands.get(command_name.lower())
+
+                if _command:
                     instance_id = f"{command_name}-{ctx.author.id}"
                     invoked_at = time()
-                    if command[2] and instance_id in self.__command_cooldown_cache:
+                    if _command[2] and instance_id in self.__command_cooldown_cache:
                         passed = invoked_at - self.__command_cooldown_cache[instance_id]
-                        if passed < command[2]:
-                            return await execute_listener("on_cooldown_trigger", ctx, command_name, command[0], command[2] - passed)
-                    
+                        if passed < _command[2]:
+                            return await execute_listener("on_cooldown_trigger", ctx, command_name, _command[0],
+                                                          _command[2] - passed)
+
                     arguments = []
                     params = {}
-                    parameters = list(signature(command[0]).parameters.items())
-                    if not command[1]:
+                    parameters = list(signature(_command[0]).parameters.items())
+                    if not _command[1]:
                         arguments.append(self)
                         parameters.pop(0)
 
                     if parameters:
                         arguments.append(ctx)
                         parameters.pop(0)
-                        
+
                         for idx, (key, param) in enumerate(parameters):
                             if idx + 1 > len(args) and param.default != Parameter.empty:
                                 params[key] = param.default
@@ -189,18 +205,19 @@ class DogeClient(Client):
 
                                 if param.kind == param.KEYWORD_ONLY and len(args) - idx - 1 != 0:
                                     value = " ".join(args[idx::])
-                                    
+
                             value = value.strip()
 
-                            if value and param.annotation and hasattr(param.annotation, "convert") and callable(param.annotation.convert):
+                            if value and param.annotation and hasattr(param.annotation, "convert") and callable(
+                                    param.annotation.convert):
                                 value = await param.annotation.convert(ctx, param, value)
                             else:
-                                value = Convertor._handle_basic_types(param, value)
+                                value = Convertor.handle_basic_types(param, value)
 
                             params[key] = value
                         self.__command_cooldown_cache[instance_id] = invoked_at
                     try:
-                        asyncio.ensure_future(command[0](*arguments, **params))
+                        asyncio.ensure_future(_command[0](*arguments, **params))
                     except TypeError:
                         raise NotEnoughArguments(
                             f"Not enough arguments were provided in command `{command_name}`.")
@@ -210,11 +227,11 @@ class DogeClient(Client):
 
             info("Dogehouse: Starting event listener loop")
             while self.__active:
-                res = loads(await self.__socket.recv())
+                res: Dict[str, Union[Dict, Any]] = loads(await self.__socket.recv())
                 op = res if isinstance(res, str) else res.get("op")
                 if op == "auth-good":
                     info("Dogehouse: Received client ready")
-                    self.user = User.from_dict(res["d"]["user"])
+                    self.user = User.from_dict(dict(res["d"]["user"]))
                     await execute_listener("on_ready")
                 elif op == "new-tokens":
                     info("Dogehouse: Received new authorization tokens")
@@ -248,7 +265,7 @@ class DogeClient(Client):
                     # for user in self.room.users:
                     #     if not isinstance(user, User):
                     #         await self.__fetch("get_user_profile", dict(userId=user.id))
-                    
+
                     # TODO: Check if joined as speaker 
                     await execute_listener("on_room_join", False)
                 elif op == "new_user_join_room":
@@ -267,9 +284,9 @@ class DogeClient(Client):
                         continue
 
                     try:
-                        async def handle_command(prefix: str):
-                            if msg.content.startswith(prefix) and len(msg.content) > len(prefix) + 1:
-                                splitted = msg.content[len(prefix)::].split(" ")
+                        async def handle_command(_prefix: str):
+                            if msg.content.startswith(_prefix) and len(msg.content) > len(_prefix) + 1:
+                                splitted = msg.content[len(_prefix)::].split(" ")
                                 await execute_command(splitted[0], Context(self, msg), *splitted[1::])
                                 return True
                             return False
@@ -280,24 +297,51 @@ class DogeClient(Client):
                         for prefix in prefixes:
                             if await handle_command(prefix):
                                 break
-                    except Exception as e:
+                    except Exception as err:
                         if "on_error" not in self.__listeners:
                             print_exc()
-                        else: 
-                            await execute_listener("on_error", e)
+                        else:
+                            await execute_listener("on_error", err)
                 elif op == "message_deleted":
                     await execute_listener("on_message_delete", res["d"]["deleterId"], res["d"]["messageId"])
                 elif op == "speaker_added":
-                    await execute_listener("on_speaker_add", res["d"]["userId"], res["d"]["roomId"], res["d"]["muteMap"])
+                    for user in self.room.users:
+                        if user.id == res["d"]["userId"] and user.current_room_id == res["d"]["roomId"]:
+                            user.room_permissions.is_speaker = True
+                            await execute_listener("on_speaker_add", user, res["d"]["muteMap"])
+                            break
                 elif op == "speaker_removed":
-                    await execute_listener("on_speaker_delete", res["d"]["userId"], res["d"]["roomId"], res["d"]["muteMap"], res["d"]["raiseHandMap"])
+                    for user in self.room.users:
+                        if user.id == res["d"]["userId"] and user.current_room_id == res["d"]["roomId"]:
+                            user.room_permissions.is_speaker = False
+                            await execute_listener("on_speaker_delete", user, res["d"]["muteMap"],
+                                                   res["d"]["raiseHandMap"])
+                            break
                 elif op == "chat_user_banned":
                     await execute_listener("on_user_ban", res["d"]["userId"])
                 elif op == "hand_raised":
                     await execute_listener("on_speaker_request", res["d"]["userId"], res["d"]["roomId"])
                 elif op == "get_current_room_users_done":
                     self.room.users = list(map(User.from_dict, res["d"]["users"]))
+                    for user in self.room.users:
+                        if user.id == self.room.creator_id:
+                            user.room_permissions.is_admin = True
                     await execute_listener("on_room_users_fetch")
+                elif op == "mod_changed" or op == "new_room_creator":
+                    changed_permission_type = "mod" if op == "mod_changed" else "admin"
+                    attribute = f"is_{changed_permission_type}"
+
+                    if changed_permission_type == "admin":
+                        for user in self.room.users:
+                            if user.room_permissions.is_admin:
+                                user.room_permissions.is_admin = False
+                                await execute_listener("on_permission_change", user, changed_permission_type)
+
+                    for user in self.room.users:
+                        if user.id == res["d"]["userId"] and user.current_room_id == res["d"]["roomId"]:
+                            setattr(user.room_permissions, attribute, not getattr(user.room_permissions, attribute))
+                            await execute_listener("on_permissions_change", user, changed_permission_type)
+                            break
 
         async def heartbeat():
             debug("Dogehouse: Starting heartbeat")
@@ -312,10 +356,12 @@ class DogeClient(Client):
                 await asyncio.sleep(topPublicRoomsInterval)
 
         async def perform_telemetry():
-            debug(f"Dogehouse: {'Starting to perform telemetry' if self.telemetry else 'Passing telemetry, as its disabled'}")
-            while self.telemetry and self.__active:
-                await self.telemetry.transmit(self)
-                await asyncio.sleep(telemetryInterval)
+            if self.telemetry:
+                debug("Dogehouse: Starting to perform telemetry")
+
+                while self.telemetry and self.__active:
+                    await self.telemetry.transmit(self)
+                    await asyncio.sleep(telemetryInterval)
 
         try:
             info("Dogehouse: Connecting with Dogehouse websocket")
@@ -324,7 +370,7 @@ class DogeClient(Client):
                 self.__active = True
                 self.__socket = ws
 
-                info("Dogehouse: Attemting to authenticate")
+                info("Dogehouse: Attempting to authenticate")
                 await self.__send('auth', {
                     "accessToken": self.__token,
                     "refreshToken": self.__refresh_token,
@@ -346,7 +392,7 @@ class DogeClient(Client):
             info("Dogehouse: Websocket connection closed peacefully")
             self.__active = False
         except ConnectionClosedError as e:
-            if (e.code == 4004):
+            if e.code == 4004:
                 raise InvalidAccessToken()
 
     def run(self):
@@ -393,6 +439,7 @@ class DogeClient(Client):
 
             client.run()
         """
+
         def decorator(func: Awaitable):
             self.__listeners[str(name if name else func.__name__).lower()] = [
                 func, True]
@@ -426,6 +473,7 @@ class DogeClient(Client):
 
             client.run()
         """
+
         def decorator(func: Awaitable):
             self.__commands[str(name if name else func.__name__).lower()] = [
                 func, True]
@@ -452,24 +500,25 @@ class DogeClient(Client):
         Args:
             name (str): The name for room.
             description (str): The description for the room.
-            public (bool, optional): Wether or not the room should be publicly visible. Defaults to True.
+            public (bool, optional): Whether or not the room should be publicly visible. Defaults to True.
         """
         if 2 <= len(name) <= 60:
-            return await self.__fetch("create_room", dict(name=str(name), description=str(description), privacy="public" if public else "private"))
+            return await self.__fetch("create_room", dict(name=str(name), description=str(description),
+                                                          privacy="public" if public else "private"))
 
         raise InvalidSize(
             "The `name` property length should be 2-60 characters long.")
 
-    async def join_room(self, id: str) -> None:
+    async def join_room(self, room_id: str) -> None:
         """
         Send a request to join a room as a listener.
 
         Args:
-            id (str): The ID of the room you want to join.
+            room_id (str): The ID of the room you want to join.
         """
-        await self.__send("join_room", dict(roomId=str(id)))
+        await self.__send("join_room", dict(roomId=str(room_id)))
 
-    async def send(self, message: str, *, whisper: List[str] = []) -> None:
+    async def send(self, message: str, *, whisper: List[str] = None) -> None:
         """
         Send a message to the current room.
 
@@ -480,10 +529,13 @@ class DogeClient(Client):
         Raises:
             NoConnectionException: Gets thrown when the client hasn't joined a room yet.
         """
+
         if not self.room:
             raise NoConnectionException("No room has been joined yet!")
 
-        await self.__send("send_room_chat_msg", dict(whisperedTo=list(map(str, whisper)), tokens=parse_message(message)))
+        await self.__send("send_room_chat_msg",
+                          dict(whisperedTo=list(map(str, [] if whisper is None else whisper)),
+                               tokens=parse_message(message)))
 
     async def ask_to_speak(self):
         """
@@ -505,19 +557,20 @@ class DogeClient(Client):
         """
         await self.__send("change_mod_status", dict(userId=str(user if isinstance(user, str) else user.id), value=True))
 
-    async def unmod(self, user: Union[str, User, BaseUser, UserPreview]):
+    async def un_mod(self, user: Union[str, User, BaseUser, UserPreview]):
         """
         Remove a user their room moderator permissions.
 
         Args:
             user (Union[str, User, BaseUser, UserPreview]): The user from which his permissions should be taken.
         """
-        await self.__send("change_mod_status", dict(userId=str(user if isinstance(user, str) else user.id), value=False))
+        await self.__send("change_mod_status",
+                          dict(userId=str(user if isinstance(user, str) else user.id), value=False))
 
     async def make_admin(self, user: Union[str, User, BaseUser, UserPreview]):
         """
         Make a user the room administrator/owner.
-        NOTE: This action is irreversable.
+        NOTE: This action is irreversible.
 
         Args:
             user (Union[str, User, BaseUser, UserPreview]): The user which should be promoted to room admin.
@@ -529,7 +582,8 @@ class DogeClient(Client):
         Force a user to be a listener.
 
         Args:
-            user (Union[str, User, BaseUser, UserPreview], optional): The user which should become a Listener. Defaults to the client.
+            user (Union[str, User, BaseUser, UserPreview], optional): The user which should become a Listener.
+                                                                      Defaults to the client.
         """
         if not user:
             user = self.user
@@ -561,7 +615,8 @@ class DogeClient(Client):
         Args:
             user (Union[str, User, BaseUser, UserPreview]): The user who should be unbanned.
         """
-        await self.__send("unban_from_room", dict(userId=str((user if isinstance(user, str) else user.id)), fetch_id=uuid4()))
+        await self.__send("unban_from_room",
+                          dict(userId=str((user if isinstance(user, str) else user.id)), fetch_id=uuid4()))
 
     async def add_speaker(self, user: Union[str, User, BaseUser, UserPreview]):
         """
@@ -572,25 +627,27 @@ class DogeClient(Client):
         """
         await self.__send("add_speaker", dict(userId=str(user if isinstance(user, str) else user.id)))
 
-    async def delete_message(self, id: str, user_id: str):
+    async def delete_message(self, message_id: str, user_id: str):
         """
         Deletes a message that has been sent by a user.
 
         Args:
-            id (str): The id of the message that should be removed.
+            message_id (str): The id of the message that should be removed.
             user_id (str): The author of that message.
         """
-        await self.__send("delete_room_chat_message", dict(messageId=str(id), userId=str(user_id)))
+        await self.__send("delete_room_chat_message", dict(messageId=str(message_id), userId=str(user_id)))
 
-    async def wait_for(self, event: str, *, timeout: float = 60.0, check: callable = None, tick: float = 0.5, fetch_arguments: Tuple[Any] = None) -> Union[Any, Tuple[Any]]:
+    async def wait_for(self, event_name: str, *, timeout: float = 60.0, check: callable = None, tick: float = 0.5,
+                       fetch_arguments: Tuple[Any] = None) -> Union[Any, Tuple[Any]]:
         """
         Manually wait for an event.
 
         Args:
-            event (str): The `on_...` event that should be waited for. (without the `on_` part)
+            event_name (str): The `on_...` event that should be waited for. (without the `on_` part)
             timeout (float, optional): How long the client will wait for a response.. Defaults to 60.0.
-            check (callable, optional): A check which will be checked for the reponse. Defaults to None.
-            tick (float, optional): The tickrate for the fetch check iteration. Defaults to 0.5.
+            check (callable, optional): A check which will be checked for the response. Defaults to None.
+            tick (float, optional): The tick-rate for the fetch check iteration. Defaults to 0.5.
+            fetch_arguments (tuple, optional): A fetch that should be executed before the wait.
 
         Raises:
             asyncio.TimeoutError: Gets thrown when the timeout has been reached.
@@ -599,8 +656,8 @@ class DogeClient(Client):
             Union[Any, Tuple[Any]]: The parameter(s) of the event.
         """
         fetch_id = str(uuid4())
-        self.__waiting_for[event] = [*self.__waiting_for[event],
-                                     fetch_id] if event in self.__waiting_for else [fetch_id]
+        self.__waiting_for[event_name] = [*self.__waiting_for[event_name],
+                                          fetch_id] if event_name in self.__waiting_for else [fetch_id]
 
         if fetch_arguments:
             await self.__fetch(*fetch_arguments)
@@ -611,9 +668,9 @@ class DogeClient(Client):
             passed += tick
 
             if passed > timeout:
-                self.__waiting_for[event].remove(fetch_id)
+                self.__waiting_for[event_name].remove(fetch_id)
                 raise asyncio.TimeoutError(
-                    f"wait_for event timed out (for `{event}`)")
+                    f"wait_for event timed out (for `{event_name}`)")
             elif fetch_id in self.__waiting_for_fetches:
                 data = self.__waiting_for_fetches[fetch_id]
                 if check is not None:
@@ -671,11 +728,12 @@ class DogeClient(Client):
         argument = argument.lower()
 
         def check(attribute: str):
-            for user in self.room.users:
-                if hasattr(user, attribute):
-                    attr = getattr(user, attribute)
-                    if hasattr(attr, "lower") and callable(getattr(attr, "lower", False)) and attr.lower() == argument:
-                        return user
+            for _user in self.room.users:
+                if hasattr(_user, attribute):
+                    attrs = getattr(_user, attribute)
+                    if hasattr(attrs, "lower") and callable(getattr(attrs, "lower", False)) \
+                            and attrs.lower() == argument:
+                        return _user
 
         for attr in ["id", "username", "displayname"]:
             user = check(attr)
